-- ElvUI Companions Datatext <Pets Datatext>
-- By Lockslap, US - Bleeding Hollow
local E, L, V, P, G	= unpack(ElvUI)
local DT			= E:GetModule("DataTexts")
local LibQTip		= LibStub("LibQTip-1.0")

local join		= string.join
local split		= string.split
local wipe		= table.wipe
local random	= math.random

local Tip
local displayString = ""
local colorString = "|cff%s%s|r"

local function ToHex(color)
	return ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

local function OnEvent(self, event, ...)
	self.text:SetFormattedText(displayString, GetNumCompanions("CRITTER"))
end

local function OnUpdate(self, elapsed)
	self.text:SetFormattedText(displayString, GetNumCompanions("CRITTER"))
end

local function PetClick(frame, info, button)
	local id, active = split(":", info)
	if tonumber(active) == 1 then
		DismissCompanion("CRITTER")
	else
		CallCompanion("CRITTER", tonumber(id))
	end
	Tip:Hide()
end

local function Click(self, button)
	Tip:Hide()
	if button == "RightButton" then
		CallCompanion("CRITTER", random(1, GetNumCompanions("CRITTER")))
	else
		ToggleSpellBook("companions")
	end
end

local function OnEnter(self)
	if InCombatLockdown() then return end
	local mounts = {}
	
	if LibQTip:IsAcquired("ElvUI_Pets") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("ElvUI_Pets", mountdb.columns)
		Tip:SmartAnchorTo(self)
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(self) self:SetTemplate("Transparent") end)
	end
	
	line = Tip:AddHeader()
	Tip:SetCell(line, 1, "|cffffffffNon-Combat Pets|r", "LEFT")
	Tip:AddSeparator()
	
	if GetNumCompanions("CRITTER") == 0 then
		line = Tip:AddLine()
		Tip:SetCell(line, 1, "|cffff0000No Pets Found!|r", "CENTER")
	else
		wipe(mounts)
		
		for i = 1, GetNumCompanions("CRITTER") do
			local _, name, spell, _, active, flags = GetCompanionInfo("CRITTER", i)
			local colored
			
			colored = colorString:format(ToHex(active == 1 and mountdb.selectColor or mountdb.petColor), name)
			
			mounts[#mounts + 1] = {
				id		= i,
				raw		= name,
				name	= colored,
				active	= active
			}
		end
		
		if #mounts == 0 then return end
		sort(mounts, function(a, b) return a["raw"] < b["raw"] end)
	
		local i = 1
		for index, value in pairs(mounts) do
			if i == 1 then line = Tip:AddLine() end
			Tip:SetCell(line, i, value.name, "LEFT", nil, nil, nil, nil, nil, 200)
			Tip:SetCellScript(line, i, "OnMouseUp", PetClick, tostring(value.id) .. ":" .. tostring(value.active))
			
			if i == mountdb.columns then
				i = 1
			else
				i = i + 1
			end
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffffPets:|r", " ", hex, "%d|r")
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

DT:RegisterDatatext("Pets", {"PLAYER_ENTERING_WORLD"}, OnEvent, OnUpdate, Click, OnEnter)